#include <p24fxxxx.h>
#include <ports.h>
#include <timer.h>
#include ".\..\App1632_LCM.h"
#include <pps.h>
#include <uart.h>
#include <stdio.h>

/* Configuration Settings */
// OSC Input = XT + PLL
// CPU Clock =  ( 8000000 * 4 ) / 1 
// Disabld WDT , ICSP/ICD = PGC2 , PGD2 
_CONFIG1( JTAGEN_OFF & GCP_OFF & GWRP_OFF & ICS_PGx2 &
		  FWDTEN_OFF & WINDIS_OFF & FWPSA_PR128 & WDTPS_PS32768 )

_CONFIG2( IESO_OFF & FNOSC_PRIPLL & 
		  FCKSM_CSDCMD & OSCIOFNC_OFF & IOL1WAY_OFF & I2C2SEL_PRI & POSCMOD_XT )

_CONFIG3( WPEND_WPSTARTMEM & WPCFG_WPCFGDIS & WPDIS_WPDIS & WPFP_WPFP511 )

#define SystemFrequency		( ( 8000000L * 4 ) / 1 ) / 2
#define Timer1Tick			( SystemFrequency / 256 / Timer1TogglesPerSec )
#define Timer1TogglesPerSec	1
#define UART2Period 		( ( SystemFrequency / UART2Baud ) / 16 ) - 1
#define UART2Baud 			19200

void UART2_PutROMString( unsigned char * String );

const unsigned char LCMString1[ ] = "PIC24FJ256GA110 "; 
const unsigned char LCMString2[ ] = "Rx:             "; 
unsigned char UARTString1[ 32 ]; 
unsigned char Timer1TickFlag = 0;
unsigned char UART_IsReceived = 0;
unsigned char UART_ReceiveBuffer = 0;
unsigned char Hours = 0 , Mins = 0 , Secs = 0;

void __attribute__( ( interrupt , no_auto_psv ) ) _T1Interrupt( void )
{
	IFS0bits.T1IF = 0;

	Timer1TickFlag = 1;

	mPORTAToggleBits( 0xff );
}

void __attribute__( ( interrupt , no_auto_psv ) ) _U2RXInterrupt( void )
{
	// MUART_XСC
	IFS1bits.U2RXIF = 0;

	UART_ReceiveBuffer = ReadUART2( );

	UART_IsReceived = 1;
}

int main( void )
{
	unsigned char Cursor = 0;

	// CPU Clock = PB Clock / 1
	CLKDIVbits.DOZEN = 0;
 
	// LEDI/O Port]wƦXAëwI/O PortAC
	mPORTAConfig( 0x00 );
	mPORTAWrite( 0x0f );

	LCM_Init( );

	LCM_SetCursor( 0 , 0 );
	LCM_PutROMString( LCMString1 );
	LCM_SetCursor( 0 , 1 );
	LCM_PutROMString( LCMString2 );

	OpenTimer1( T1_ON & T1_IDLE_CON & T1_GATE_OFF & T1_PS_1_256 & T1_SYNC_EXT_OFF & T1_SOURCE_INT ,
			    Timer1Tick );

	ConfigIntTimer1( T1_INT_ON & T1_INT_PRIOR_4 );

	// ]wCСAXrC
	LCM_SetCursor( 0 , 0 );
	LCM_PutROMString( LCMString1 );
	LCM_SetCursor( 0 , 1 );
	LCM_PutROMString( LCMString2 );

	PPSUnLock;								// __builtin Function
	PPSInput( PPS_U2RX , PPS_RP10 );
	PPSOutput( PPS_RP17 ,PPS_U2TX );
	PPSLock;								// __builtin Function

	OpenUART2( UART_EN & UART_IDLE_CON & UART_EN_WAKE & UART_DIS_LOOPBACK & UART_DIS_ABAUD &
			   UART_NO_PAR_8BIT & UART_1STOPBIT & UART_IrDA_DISABLE & UART_MODE_SIMPLEX &
			   UART_UEN_00 & UART_UXRX_IDLE_ONE & UART_BRGH_SIXTEEN , 
			   UART_INT_TX_EACH_CHAR & UART_IrDA_POL_INV_ZERO & UART_SYNC_BREAK_DISABLED & UART_TX_ENABLE &
			   UART_INT_RX_CHAR & UART_ADR_DETECT_DIS & UART_RX_OVERRUN_CLEAR ,
			   UART2Period );

	ConfigIntUART2( UART_RX_INT_EN & UART_RX_INT_PR4 & UART_TX_INT_DIS & UART_TX_INT_PR4 ); 

	Cursor = 3;

	while( 1 )
	{
		// UART ?
		if( UART_IsReceived )
		{
			UART_IsReceived = 0;

			// ]wCСAXrC
			LCM_SetCursor( Cursor++ , 1 );
			LCM_PutASCII( UART_ReceiveBuffer );

			if( Cursor > 15 ) Cursor = 3;
		}

		// BzC
		if( Timer1TickFlag ) 	
		{
			if( ++Secs > 59 ) 
			{
				Secs = 0;
				if( ++Mins > 59 )
				{
					Mins = 0;
					if( ++Hours > 23 )
						Hours = 0;
				}
			}

			sprintf( ( char * ) UARTString1 , "Now Time %2d:%2d:%2d\r" ,
					 Hours , Mins , Secs );

			UART2_PutROMString( UARTString1 );

			Timer1TickFlag = 0;
		}
	}

	while( 1 );
}

void UART2_PutROMString( unsigned char * String ) 
{
	while( *String != 0x00 )
	{
		while( BusyUART2( ) );
	    WriteUART2( *String++ );
	}
}
