#include <p24fxxxx.h>
#include <ports.h>
#include <timer.h>
#include ".\..\App1632_LCM.h"
#include <pps.h>
#include <rtcc.h>

/* Configuration Settings */
// OSC Input = XT + PLL
// CPU Clock =  ( 8000000 * 4 ) / 1 
// Disabld WDT , ICSP/ICD = PGC2 , PGD2 
_CONFIG1( JTAGEN_OFF & GCP_OFF & GWRP_OFF & ICS_PGx2 &
		  FWDTEN_OFF & WINDIS_OFF & FWPSA_PR128 & WDTPS_PS32768 )

_CONFIG2( IESO_OFF & FNOSC_PRIPLL & 
		  FCKSM_CSDCMD & OSCIOFNC_OFF & IOL1WAY_OFF & I2C2SEL_PRI & POSCMOD_XT )

_CONFIG3( WPEND_WPSTARTMEM & WPCFG_WPCFGDIS & WPDIS_WPDIS & WPFP_WPFP511 )

#define SystemFrequency		( ( 8000000L * 4 ) / 1 ) / 2
#define Timer1Tick			( SystemFrequency / 256 / Timer1TogglesPerSec )
#define Timer1TogglesPerSec	10

const unsigned char LCMString1[ ] = "PIC24FJ256GA110 "; 
const unsigned char LCMString2[ ] = "Time:"; 

unsigned char Timer1Flag = 0;

void __attribute__( ( interrupt , no_auto_psv ) ) _T1Interrupt( void )
{
	IFS0bits.T1IF = 0;

	Timer1Flag = 1;

	mPORTAToggleBits( 0xff );
}

int main( void )
{
	rtccTime DefaultTime; 
	rtccTime NowTime; 

	// CPU Clock = PB Clock / 1
	CLKDIVbits.DOZEN = 0;

	// LEDI/O Port]wƦXAëwI/O PortAC
	mPORTAConfig( 0x00 );
	mPORTAWrite( 0x0f );

	LCM_Init( );

	LCM_SetCursor( 0 , 0 );
	LCM_PutROMString( LCMString1 );
	LCM_SetCursor( 0 , 1 );
	LCM_PutROMString( LCMString2 );

	OpenTimer1( T1_ON & T1_IDLE_CON & T1_GATE_OFF & T1_PS_1_256 & T1_SYNC_EXT_OFF & T1_SOURCE_INT ,
			    Timer1Tick );

	ConfigIntTimer1( T1_INT_ON & T1_INT_PRIOR_4 );

	DefaultTime.f.hour = 0x23;
	DefaultTime.f.min = 0x59;
	DefaultTime.f.sec = 0x50;
	RtccWriteTime( &DefaultTime , 1 );

	RtccInitClock( );
 	RtccWrOn( );
	mRtccOn( );
	mRtccWrOff( );

	while( 1 )
	{
		if( Timer1Flag )
		{
			Timer1Flag = 0;

			RtccReadTime( &NowTime );
			LCM_SetCursor( 7 , 1 );

			LCM_PutASCII( ( ( NowTime.f.hour >> 4 ) & 0x0f ) + '0' ); 
			LCM_PutASCII( ( NowTime.f.hour & 0x0f ) + '0' ); 
			LCM_PutASCII( ':' );
			LCM_PutASCII( ( ( NowTime.f.min >> 4 ) & 0x0f ) + '0' ); 
			LCM_PutASCII( ( NowTime.f.min & 0x0f ) + '0' ); 
			LCM_PutASCII( ':' );
			LCM_PutASCII( ( ( NowTime.f.sec >> 4 ) & 0x0f ) + '0' ); 
			LCM_PutASCII( ( NowTime.f.sec & 0x0f ) + '0' ); 
		}
	}

	while( 1 );
}
