#include <p24fxxxx.h>
#include <ports.h>
#include <timer.h>
#include ".\..\App1632_LCM.h"
#include <pps.h>
#include <outcompare.h>

/* Configuration Settings */
// OSC Input = XT + PLL
// CPU Clock =  ( 8000000 * 4 ) / 1 
// Disabld WDT , ICSP/ICD = PGC2 , PGD2 
_CONFIG1( JTAGEN_OFF & GCP_OFF & GWRP_OFF & ICS_PGx2 &
		  FWDTEN_OFF & WINDIS_OFF & FWPSA_PR128 & WDTPS_PS32768 )

_CONFIG2( IESO_OFF & FNOSC_PRIPLL & 
		  FCKSM_CSDCMD & OSCIOFNC_OFF & IOL1WAY_OFF & I2C2SEL_PRI & POSCMOD_XT )

_CONFIG3( WPEND_WPSTARTMEM & WPCFG_WPCFGDIS & WPDIS_WPDIS & WPFP_WPFP511 )

#define SystemFrequency		( ( 8000000L * 4 ) / 1 ) / 2
#define Timer1Tick			( SystemFrequency / 256 / Timer1TogglesPerSec )
#define Timer1TogglesPerSec	10
#define OC1Period			( SystemFrequency / 1 / OC1TogglesPerSec )
#define OC1TogglesPerSec	10000L

const unsigned char LCMString1[ ] = "PIC24FJ256GA110 "; 
const unsigned char LCMString2[ ] = "Duty:   %";

unsigned char Timer1TickFlag = 0;

void __attribute__( ( interrupt , no_auto_psv ) ) _T1Interrupt( void )
{
	IFS0bits.T1IF = 0;

	Timer1TickFlag = 1;

	mPORTAToggleBits( 0x01 );
}

int main( void )
{
	unsigned long Temporary;

	// CPU Clock = PB Clock / 1
	CLKDIVbits.DOZEN = 0;

	// LEDI/O Port]wƦXAëwI/O PortAC
	mPORTAConfig( 0xfe );
	mPORTAWrite( 0x01 );

	LCM_Init( );

	LCM_SetCursor( 0 , 0 );
	LCM_PutROMString( LCMString1 );
	LCM_SetCursor( 0 , 1 );
	LCM_PutROMString( LCMString2 );

	OpenTimer1( T1_ON & T1_IDLE_CON & T1_GATE_OFF & T1_PS_1_256 & T1_SYNC_EXT_OFF & T1_SOURCE_INT ,
			    Timer1Tick );

	ConfigIntTimer1( T1_INT_ON & T1_INT_PRIOR_4 );

	// RP11 -> ( RD0 ) Pin 72
	PPSUnLock;								// __builtin Function
	PPSOutput( PPS_RP11 , PPS_OC1 );
	PPSLock;								// __builtin Function

	OpenOC1_GB( OC_IDLE_CON & OC_SYSCLK_SRC & OC_FAULT0_IN_DISABLE &
				OC_PWM_FAULT_CLEAR & OC_TRIG_CLEAR_SW & OC_PWM_EDGE_ALIGN ,
				OC_FAULT_MODE_PWM_CYCLE & OC_PWM_FAULT_OUT_LOW & OC_FAULT_PIN_UNCHANGE & OC_OUT_NOT_INVERT &
				OC_CASCADE_DISABLE & OC_SYNC_ENABLE & OC_UNTRIGGER_TIMER & OC_DIRN_OUTPUT & OC_SYNC_TRIG_IN_OC1 ,
				OC1Period , 0 );

	while( 1 )
	{
		if( Timer1TickFlag )
		{
			Timer1TickFlag = 0;

			OC1R += ( OC1Period / 10 );
	
			if( OC1R > OC1Period )
				OC1R = 0;

			Temporary = OC1R * 100L;
			Temporary = Temporary / OC1Period;
	
			LCM_SetCursor( 5 , 1 );
			LCM_PutNumber( Temporary , 3 );
		}
	}

	while( 1 );
}
